/////////////////////////////////////////////////////////////////////////////
//
//	Project		:	Active Commentary v1.0 (Visual C++ Dev Studio AddIn)
//	File		:	Commands.h
//	Description	:	Interfaces for addin commands 
//					(CCommands class, CProjectData class, CComment class)
//
//	Created and modified by Andrey Levin
//	mail to: Levin@artis-inventum.de
/////////////////////////////////////////////////////////////////////////////

#if !defined(AFX_COMMANDS_H__75E9F7EC_86F0_11D4_A165_0050DA5F60A9__INCLUDED_)
#define AFX_COMMANDS_H__75E9F7EC_86F0_11D4_A165_0050DA5F60A9__INCLUDED_

#include "ActiveCommentsTypes.h"
#include <afxtempl.h>

class CComment;
class CProjectData;
/////////////////////////////////////////////////////
// Global Section
#define AC_CURRENTVERSION	100
#define AC_WORKINGSTRING "({[String For Active Comment Search : 1234567890)}]"
#define AC_PATH "\\AComment\\"
#define EXTR_WITHOUT_EXT		0
#define EXTR_WITHOUT_PATH		1
#define EXTR_WITHOUT_PATHANDEXT	2
#define SEARCH_SELECTIONISEMPTY	0
#define SEARCH_ISOK				1
#define SEARCH_ISNO				2

CString GetFileExt(LPCSTR FileName);
CString	GetFilePath(LPCSTR FileName);
CString GetProjectName();
CString GetTitleFromSelection(ITextSelection* pSel);
CString GetTitleFromString(CString FromStr);
void	StripString(CString & s, const char *symbols  =" ");
BOOL	SetSelection(long row, long col, CString selection,ITextSelection* pSel);
BOOL	SelectComment(CString Title, ITextSelection *pSel);
BOOL	OpenDocument(IApplication *pApp, CString DocName);
BOOL	GetCurrentDocument(IApplication *pApp,IGenericDocument*&pDoc);
BOOL	GetCurrentSelection(IGenericDocument* pDoc, ITextSelection*&pSel);
void	ExecuteLink(CComment *pComment);
void	ExtractName(CString &FileName, int mode);
long	FindCommentaryByTitle(ITextSelection* pSel, CString Title);
BOOL	AddToHistory(CStringArray	*pArr, CString Str);
void	FillCodeTemplList(CMapStringToString *pMap);
DWORD	IsSelectionInTheCommentBlock(ITextSelection *pSel);

/////////////////////////////////////////////////////
// Class CComment 
class CComment : public CObject
{
DECLARE_SERIAL(CComment)
public:
	CComment();
	CComment(long Time,CString DocumentName, CString Link, CString Protocol);
	~CComment();
	long	m_nTimeOfCreate;
	CString	m_szDocumentName;
	CString	m_szLink;
	CString	m_szProtocol;

	void Serialize(CArchive &ar);
};

/////////////////////////////////////////////////////
// Class CProjectData 
class CProjectData : public CObject
{
DECLARE_SERIAL(CProjectData)
public: 
	CProjectData();
	~CProjectData();
	CMapStringToOb	m_CurrentProject;
	CStringArray	m_History;
	int				m_nVersion;

	BOOL DeleteMaps();
	void Serialize(CArchive &ar);
	BOOL AddToHistory(CString Str);
};

/////////////////////////////////////////////////////
// Class CCommands 
class CCommands : 
	public CComDualImpl<ICommands, &IID_ICommands, &LIBID_ActiveComments>, 
	public CComObjectRoot,
	public CComCoClass<CCommands, &CLSID_Commands>
{
protected:
	IApplication* m_pApplication;
	
public:
	CCommands();
	~CCommands();
	void SetApplicationObject(IApplication* m_pApplication);
	IApplication* GetApplicationObject() { return m_pApplication; }
	void UnadviseFromEvents();

	CMapStringToOb	*m_pCurrentDocument;
	CProjectData	m_Project;
	BOOL			m_bFreeFromAConrtol;

	BEGIN_COM_MAP(CCommands)
		COM_INTERFACE_ENTRY(IDispatch)
		COM_INTERFACE_ENTRY(ICommands)
	END_COM_MAP()
	DECLARE_NOT_AGGREGATABLE(CCommands)

public:
// ICommands methods
	STDMETHOD(CommentSetup)();
	STDMETHOD(CommentView)();
	STDMETHOD(CommentDelete)();
	STDMETHOD(About)();
	STDMETHOD(UpperCase)();
	STDMETHOD(LowerCase)();
	STDMETHOD(Capitalize)();
	STDMETHOD(ConvertSpaceToTab)();
	STDMETHOD(ViewAllComments)();
	STDMETHOD(CommentClear)();
	STDMETHOD(AddCodeTemplate)();

protected:
	CComment		*m_pComment;
	BOOL			CheckMap();
protected:
	// This class template is used as the base class for the Application
	//  event handler object and the Debugger event handler object,
	//  which are declared below.
	template <class IEvents, const IID* piidEvents, const GUID* plibid,
		class XEvents, const CLSID* pClsidEvents>

	class XEventHandler :
		public CComDualImpl<IEvents, piidEvents, plibid>,
		public CComObjectRoot,
		public CComCoClass<XEvents, pClsidEvents>
	{
	public:
		BEGIN_COM_MAP(XEvents)
			COM_INTERFACE_ENTRY(IDispatch)
			COM_INTERFACE_ENTRY_IID(*piidEvents, IEvents)
		END_COM_MAP()
		DECLARE_NOT_AGGREGATABLE(XEvents)
		void Connect(IUnknown* pUnk)
		{ VERIFY(SUCCEEDED(AtlAdvise(pUnk, this, *piidEvents, &m_dwAdvise))); }
		void Disconnect(IUnknown* pUnk)
		{ AtlUnadvise(pUnk, *piidEvents, m_dwAdvise); }

		CCommands* m_pCommands;

	protected:
		DWORD m_dwAdvise;
	};

	// This object handles events fired by the Application object
	class XApplicationEvents : public XEventHandler<IApplicationEvents, 
		&IID_IApplicationEvents, &LIBID_ActiveComments, 
		XApplicationEvents, &CLSID_ApplicationEvents>
	{
	public:
		// IApplicationEvents methods
		STDMETHOD(BeforeBuildStart)(THIS);
		STDMETHOD(BuildFinish)(THIS_ long nNumErrors, long nNumWarnings);
		STDMETHOD(BeforeApplicationShutDown)(THIS);
		STDMETHOD(DocumentOpen)(THIS_ IDispatch * theDocument);
		STDMETHOD(BeforeDocumentClose)(THIS_ IDispatch * theDocument);
		STDMETHOD(DocumentSave)(THIS_ IDispatch * theDocument);
		STDMETHOD(NewDocument)(THIS_ IDispatch * theDocument);
		STDMETHOD(WindowActivate)(THIS_ IDispatch * theWindow);
		STDMETHOD(WindowDeactivate)(THIS_ IDispatch * theWindow);
		STDMETHOD(WorkspaceOpen)(THIS);
		STDMETHOD(WorkspaceClose)(THIS);
		STDMETHOD(NewWorkspace)(THIS);
	};
	typedef CComObject<XApplicationEvents> XApplicationEventsObj;
	XApplicationEventsObj* m_pApplicationEventsObj;

	// This object handles events fired by the Application object
	class XDebuggerEvents : public XEventHandler<IDebuggerEvents, 
		&IID_IDebuggerEvents, &LIBID_ActiveComments, 
		XDebuggerEvents, &CLSID_DebuggerEvents>
	{
	public:
		// IDebuggerEvents method
	    STDMETHOD(BreakpointHit)(THIS_ IDispatch * pBreakpoint);
	};
	typedef CComObject<XDebuggerEvents> XDebuggerEventsObj;
	XDebuggerEventsObj* m_pDebuggerEventsObj;

};

typedef CComObject<CCommands> CCommandsObj;

//{{AFX_INSERT_LOCATION}}
// Microsoft Visual C++ will insert additional declarations immediately before the previous line.

#endif // !defined(AFX_COMMANDS_H__75E9F7EC_86F0_11D4_A165_0050DA5F60A9__INCLUDED)
